@@ -1,4 +1,4 @@
-use crate::lexer::TokenKind;
+use crate::{kalk_value::KalkFloat, lexer::TokenKind};
 
 /// A tree structure of a statement.
 #[derive(Debug, Clone, PartialEq)]
@@ -19,7 +19,7 @@ pub enum Expr {
     Var(Identifier),
     Group(Box<Expr>),
     FnCall(Identifier, Vec<Expr>),
-    Literal(f64),
+    Literal(KalkFloat),
     Boolean(bool),
     Piecewise(Vec<ConditionalPiece>),
     Vector(Vec<Expr>),
@@ -100,16 +100,16 @@ impl Identifier {
 pub fn build_literal_ast(kalk_value: &crate::kalk_value::KalkValue) -> Expr {
     if kalk_value.has_imaginary() {
         Expr::Binary(
-            Box::new(Expr::Literal(kalk_value.to_f64())),
+            Box::new(Expr::Literal(kalk_value.to_float())),
             TokenKind::Plus,
             Box::new(Expr::Binary(
-                Box::new(Expr::Literal(kalk_value.imaginary_to_f64())),
+                Box::new(Expr::Literal(kalk_value.imaginary_to_float())),
                 TokenKind::Star,
                 Box::new(Expr::Var(Identifier::from_full_name("i"))),
             )),
         )
     } else {
-        Expr::Literal(kalk_value.to_f64())
+        Expr::Literal(kalk_value.to_float())
     }
 }
 
