@@ -2,14 +2,10 @@
 pub mod with_rug;
 
 #[cfg(feature = "rug")]
-use rug::Float;
-#[cfg(feature = "rug")]
-pub use with_rug::*;
+use rug::{ops::Pow, Float};
 
 #[cfg(not(feature = "rug"))]
 pub mod regular;
-#[cfg(not(feature = "rug"))]
-pub use regular::*;
 
 mod rounding;
 
@@ -18,8 +14,16 @@ use crate::errors::KalkError;
 use crate::radix;
 use wasm_bindgen::prelude::*;
 
+use self::rounding::EstimationResult;
+
 const ACCEPTABLE_COMPARISON_MARGIN: f64 = 0.00000001;
 
+#[cfg(feature = "rug")]
+pub(crate) type KalkFloat = rug::Float;
+
+#[cfg(not(feature = "rug"))]
+pub(crate) type KalkFloat = f64;
+
 #[macro_export]
 #[cfg(not(feature = "rug"))]
 macro_rules! float {
@@ -33,7 +37,7 @@ macro_rules! float {
 macro_rules! float {
     ($x:expr) => {{
         use rug::Float;
-        Float::with_val(63, $x)
+        Float::with_val(1024, $x)
     }};
 }
 
@@ -108,7 +112,6 @@ macro_rules! as_number_or_zero {
 #[wasm_bindgen]
 #[derive(Clone)]
 pub struct ScientificNotation {
-    pub negative: bool,
     pub value: f64,
     pub exponent: i32,
     pub imaginary: bool,
@@ -121,17 +124,43 @@ pub enum ComplexNumberType {
     Imaginary,
 }
 
+#[wasm_bindgen]
+#[derive(Clone, Copy)]
+pub enum ScientificNotationFormat {
+    Normal,
+    Engineering,
+}
+
 #[wasm_bindgen]
 impl ScientificNotation {
     #[wasm_bindgen(js_name = toString)]
     pub fn to_js_string(&self) -> String {
         self.to_string()
     }
+
+    pub fn to_string_format(&self, format: ScientificNotationFormat) -> String {
+        match format {
+            ScientificNotationFormat::Normal => self.to_string(),
+            ScientificNotationFormat::Engineering => self.to_string_eng(),
+        }
+    }
+
+    fn to_string_eng(&self) -> String {
+        let exponent = self.exponent - 1;
+        let modulo = exponent % 3;
+        let value = self.value * 10_f64.powi(modulo);
+
+        ScientificNotation {
+            value,
+            exponent: exponent - modulo + 1,
+            imaginary: self.imaginary,
+        }
+        .to_string()
+    }
 }
 
 impl std::fmt::Display for ScientificNotation {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let sign = if self.negative { "-" } else { "" };
         let digits_and_mul = if self.value == 1f64 {
             String::new()
         } else {
@@ -140,11 +169,10 @@ impl std::fmt::Display for ScientificNotation {
 
         write!(
             f,
-            "{}{}10^{} {}",
-            sign,
+            "{}10^{}{}",
             digits_and_mul,
             self.exponent - 1,
-            if self.imaginary { "i" } else { "" }
+            if self.imaginary { " i" } else { "" }
         )
     }
 }
@@ -164,10 +192,10 @@ impl std::fmt::Display for KalkValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             KalkValue::Number(real, imaginary, _) => {
-                let as_str = format_number(primitive!(real));
+                let as_str = format_number_big(real);
 
                 if self.has_imaginary() {
-                    let imaginary_as_str = format_number(primitive!(imaginary).abs());
+                    let imaginary_as_str = format_number_big(&imaginary.clone().abs());
                     let sign = if imaginary < &0f64 { "-" } else { "+" };
 
                     if &as_str == "0" {
@@ -187,12 +215,21 @@ impl std::fmt::Display for KalkValue {
                 }
             }
             KalkValue::Vector(values) => {
+                let get_estimation: fn(&KalkValue) -> String = |x| {
+                    x.estimate()
+                        .unwrap_or_else(|| EstimationResult {
+                            value: x.to_string(),
+                            is_exact: false,
+                        })
+                        .value
+                };
+
                 write!(
                     f,
                     "({})",
                     values
                         .iter()
-                        .map(|x| x.estimate().unwrap_or_else(|| x.to_string()))
+                        .map(get_estimation)
                         .collect::<Vec<String>>()
                         .join(", ")
                 )
@@ -202,7 +239,13 @@ impl std::fmt::Display for KalkValue {
                 let mut longest = 0;
                 for row in rows {
                     for value in row {
-                        let value_str = value.estimate().unwrap_or_else(|| value.to_string());
+                        let value_str = value
+                            .estimate()
+                            .unwrap_or_else(|| EstimationResult {
+                                value: value.to_string(),
+                                is_exact: false,
+                            })
+                            .value;
                         longest = longest.max(value_str.len());
                         value_strings.push(format!("{},", value_str));
                     }
@@ -288,7 +331,7 @@ impl KalkValue {
         }
     }
 
-    pub fn to_string_pretty_radix(&self, radix: u8) -> String {
+    pub fn to_string_pretty_radix(&self, radix: u8, format: ScientificNotationFormat) -> String {
         let (real, imaginary, unit) = match self {
             KalkValue::Number(real, imaginary, unit) => (real, imaginary, unit),
             _ => return self.to_string(),
@@ -308,21 +351,31 @@ impl KalkValue {
         let mut new_real = real.clone();
         let mut new_imaginary = imaginary.clone();
         let mut has_scientific_notation = false;
-        let result_str = if (-6..8).contains(&sci_notation_real.exponent) || real == &0f64 {
+        let is_engineering_mode = matches!(format, ScientificNotationFormat::Engineering);
+        let result_str = if is_engineering_mode {
+            has_scientific_notation = true;
+
+            sci_notation_real.to_string_format(ScientificNotationFormat::Engineering)
+        } else if (-6..8).contains(&sci_notation_real.exponent) || real == &0f64 {
             self.to_string_real(radix)
         } else if sci_notation_real.exponent <= -14 {
             new_real = float!(0);
+
             String::from("0")
         } else if radix == 10 {
             has_scientific_notation = true;
 
-            sci_notation_real.to_string().trim().to_string()
+            sci_notation_real.to_string_format(format)
         } else {
-            return String::new();
+            self.to_string_real(radix)
         };
 
         let sci_notation_imaginary = self.to_scientific_notation(ComplexNumberType::Imaginary);
-        let result_str_imaginary = if (-6..8).contains(&sci_notation_imaginary.exponent)
+        let result_str_imaginary = if is_engineering_mode {
+            has_scientific_notation = true;
+
+            sci_notation_imaginary.to_string_format(ScientificNotationFormat::Engineering)
+        } else if (-6..8).contains(&sci_notation_imaginary.exponent)
             || imaginary == &0f64
             || imaginary == &1f64
         {
@@ -333,9 +386,9 @@ impl KalkValue {
         } else if radix == 10 {
             has_scientific_notation = true;
 
-            sci_notation_imaginary.to_string().trim().to_string()
+            sci_notation_imaginary.to_string_format(format)
         } else {
-            return String::new();
+            self.to_string_real(radix)
         };
 
         let mut output = result_str;
@@ -365,18 +418,19 @@ impl KalkValue {
         let new_value = KalkValue::Number(new_real, new_imaginary, unit.clone());
 
         if let Some(estimate) = new_value.estimate() {
-            if estimate != output && radix == 10 {
-                output.push_str(&format!(" ≈ {}", estimate));
+            if estimate.value != output && radix == 10 {
+                let equal_sign = if estimate.is_exact { "=" } else { "≈" };
+                output.push_str(&format!(" {equal_sign} {}", estimate.value));
             }
-        } else if has_scientific_notation {
+        } else if has_scientific_notation && !is_engineering_mode {
             output.insert_str(0, &format!("{} ≈ ", self));
         }
 
         output
     }
 
-    pub fn to_string_pretty(&self) -> String {
-        self.to_string_pretty_radix(10)
+    pub fn to_string_pretty(&self, format: ScientificNotationFormat) -> String {
+        self.to_string_pretty_radix(10, format)
     }
 
     pub fn to_string_with_unit(&self) -> String {
@@ -389,7 +443,7 @@ impl KalkValue {
     }
 
     /// Get an estimate of what the number is, eg. 3.141592 => π. Does not work properly with scientific notation.
-    pub fn estimate(&self) -> Option<String> {
+    pub fn estimate(&self) -> Option<EstimationResult> {
         let rounded_real = rounding::estimate(self, ComplexNumberType::Real);
         let rounded_imaginary = rounding::estimate(self, ComplexNumberType::Imaginary);
 
@@ -398,20 +452,26 @@ impl KalkValue {
         }
 
         let mut output = String::new();
-        if let Some(value) = rounded_real {
-            output.push_str(&value);
+        let mut real_is_exact = rounded_real.is_none();
+        if let Some(result) = rounded_real {
+            real_is_exact = result.is_exact;
+            output.push_str(&result.value);
         } else if self.has_real() {
             output.push_str(&self.to_string_real(10));
         }
 
-        let imaginary_value = if let Some(value) = rounded_imaginary {
-            Some(value)
+        let mut imaginary_is_exact = rounded_imaginary.is_none();
+        let imaginary_value = if let Some(result) = rounded_imaginary {
+            imaginary_is_exact = result.is_exact;
+
+            Some(result.value)
         } else if self.has_imaginary() {
             Some(self.to_string_imaginary(10, false))
         } else {
             None
         };
 
+        let is_exact = real_is_exact && imaginary_is_exact;
         if let Some(value) = imaginary_value {
             // Clear output if it's just 0.
             if output == "0" {
@@ -422,7 +482,10 @@ impl KalkValue {
                 // If both values ended up being estimated as zero,
                 // return zero.
                 if output.is_empty() {
-                    return Some(String::from("0"));
+                    return Some(EstimationResult {
+                        value: String::from("0"),
+                        is_exact,
+                    });
                 }
             } else {
                 let sign = if value.starts_with('-') { "-" } else { "+" };
@@ -441,7 +504,10 @@ impl KalkValue {
             }
         }
 
-        Some(output)
+        Some(EstimationResult {
+            value: output,
+            is_exact,
+        })
     }
 
     /// Basic up/down rounding from 0.00xxx or 0.999xxx or xx.000xxx, etc.
@@ -516,8 +582,7 @@ impl KalkValue {
         let exponent = value.abs().log10().floor() as i32 + 1;
 
         ScientificNotation {
-            negative: value < 0f64,
-            value: value / (10f64.powf(exponent as f64 - 1f64) as f64),
+            value: value / (10f64.powf(exponent as f64 - 1f64)),
             // I... am not sure what else to do...
             exponent,
             imaginary: complex_number_type == ComplexNumberType::Imaginary,
@@ -548,7 +613,7 @@ impl KalkValue {
         if let KalkValue::Number(real, _, unit) = self {
             let result = crate::interpreter::convert_unit(
                 context,
-                &Expr::Literal(primitive!(real)),
+                &Expr::Literal(real.clone()),
                 unit.as_ref(),
                 Some(&to_unit.to_string()),
             );
@@ -906,7 +971,7 @@ impl KalkValue {
             ) => {
                 if self.has_imaginary()
                     || imaginary_rhs != &0f64
-                    || (real < 0f64 && real_rhs < &1f64)
+                    || (real < 0f64 && real_rhs.clone().abs() < 1f64)
                 {
                     let a = real;
                     let b = imaginary;
@@ -1123,6 +1188,37 @@ pub fn format_number(input: f64) -> String {
     spaced(&result)
 }
 
+#[cfg(feature = "rug")]
+pub fn format_number_big(input: &Float) -> String {
+    if input.clone().log10() < 0f64 {
+        return input.to_f64().to_string();
+    }
+
+    let input_str = input.to_string();
+    let mut result = if input_str.contains('.') {
+        input_str
+            .trim_end_matches('0')
+            .trim_end_matches('.')
+            .to_string()
+    } else {
+        input_str
+    };
+
+    if let Some(dot_index) = result.find('.') {
+        let decimal_count = result.len() - dot_index;
+        if decimal_count > 10 {
+            result = result[..(result.len() - decimal_count + 10)].to_string();
+        }
+    }
+
+    spaced(&result)
+}
+
+#[cfg(not(feature = "rug"))]
+pub fn format_number_big(input: &f64) -> String {
+    format_number(*input)
+}
+
 fn calculate_vector(
     x: KalkValue,
     y: &KalkValue,
@@ -1309,7 +1405,6 @@ fn pow(x: f64, y: f64) -> f64 {
 
 #[cfg(feature = "rug")]
 fn pow(x: Float, y: Float) -> Float {
-    use rug::ops::Pow;
     x.pow(y)
 }
 
@@ -1377,9 +1472,11 @@ impl From<i32> for KalkValue {
 
 #[cfg(test)]
 mod tests {
-    use crate::kalk_value::{spaced, KalkValue};
+    use crate::kalk_value::{spaced, KalkValue, ScientificNotationFormat};
     use crate::test_helpers::cmp;
 
+    use super::ScientificNotation;
+
     #[test]
     fn test_spaced() {
         assert_eq!(spaced("1"), String::from("1"));
@@ -1479,6 +1576,8 @@ mod tests {
             ((0f64, 2f64), (0f64, 3f64), (-0.0043748f64, 0.0078460f64)),
             ((3f64, 2f64), (0f64, 3f64), (-0.1304148f64, -0.111153f64)),
             ((3f64, 2f64), (4f64, 3f64), (28.8577819f64, -2.422530f64)),
+            ((-9f64, 0f64), (0.5f64, 0f64), (0f64, 3f64)),
+            ((-9f64, 0f64), (-0.5f64, 0f64), (0f64, -1f64 / 3f64)),
             (
                 (3f64, 0f64),
                 (0f64, 1f64 / 3f64),
@@ -1503,33 +1602,75 @@ mod tests {
     #[test]
     fn test_to_string_pretty() {
         let in_out = vec![
-            (0.99999, 0.0, "0.99999 ≈ 1"),
-            (-0.99999, 0.0, "-0.99999 ≈ -1"),
-            (0.0, 0.99999, "0.99999i ≈ i"),
-            (0.000000001, 0.0, "10^-9 ≈ 0"),
-            (0.0, 0.000000001, "10^-9 i ≈ 0"),
-            (0.99999, 0.999999, "0.99999 + 0.999999i ≈ 1 + i"),
-            (1.0, 0.99999, "1 + 0.99999i ≈ 1 + i"),
-            (-0.99999, 0.999999, "-0.99999 + 0.999999i ≈ -1 + i"),
-            (0.99999, -0.999999, "0.99999 - 0.999999i ≈ 1 - i"),
-            (-1.0, 0.99999, "-1 + 0.99999i ≈ -1 + i"),
-            (1.0, -0.99999, "1 - 0.99999i ≈ 1 - i"),
-            (-0.99999, 1.0, "-0.99999 + i ≈ -1 + i"),
-            (0.99999, -1.0, "0.99999 - i ≈ 1 - i"),
-            (0.000000001, 0.000000001, "10^-9 + 10^-9 i ≈ 0"),
-            (1.0, 0.000000001, "1 + 10^-9 i ≈ 1"),
-            (0.000000001, 1.0, "10^-9 + i ≈ i"),
-            (-1.0, 0.000000001, "-1 + 10^-9 i ≈ -1"),
-            (0.000000001, -1.0, "10^-9 - i ≈ -i"),
-            (10e-17, 1.0, "i"),
-            (1.0, 10e-17, "1"),
-            (10e-16, 0.0, "0"),
-            (3.00000000004, 0.0, "3"),
+            (float!(0.99999), float!(0.0), "0.99999 ≈ 1"),
+            (float!(0.00000001), float!(0.0), "0.00000001 ≈ 10^-8"),
+            (float!(-0.99999), float!(0.0), "-0.99999 ≈ -1"),
+            (float!(0.0), float!(0.99999), "0.99999i ≈ i"),
+            (float!(0.000000001), float!(0.0), "10^-9 ≈ 0"),
+            (float!(0.0), float!(0.000000001), "10^-9 i ≈ 0"),
+            (
+                float!(0.99999),
+                float!(0.999999),
+                "0.99999 + 0.999999i ≈ 1 + i",
+            ),
+            (float!(1.0), float!(0.99999), "1 + 0.99999i ≈ 1 + i"),
+            (
+                float!(-0.99999),
+                float!(0.999999),
+                "-0.99999 + 0.999999i ≈ -1 + i",
+            ),
+            (
+                float!(0.99999),
+                float!(-0.999999),
+                "0.99999 - 0.999999i ≈ 1 - i",
+            ),
+            (float!(-1.0), float!(0.99999), "-1 + 0.99999i ≈ -1 + i"),
+            (float!(1.0), float!(-0.99999), "1 - 0.99999i ≈ 1 - i"),
+            (float!(-0.99999), float!(1.0), "-0.99999 + i ≈ -1 + i"),
+            (float!(0.99999), float!(-1.0), "0.99999 - i ≈ 1 - i"),
+            (
+                float!(0.000000001),
+                float!(0.000000001),
+                "10^-9 + 10^-9 i ≈ 0",
+            ),
+            (float!(1.0), float!(0.000000001), "1 + 10^-9 i ≈ 1"),
+            (float!(0.000000001), float!(1.0), "10^-9 + i ≈ i"),
+            (float!(-1.0), float!(0.000000001), "-1 + 10^-9 i ≈ -1"),
+            (float!(0.000000001), float!(-1.0), "10^-9 - i ≈ -i"),
+            (float!(10e-17), float!(1.0), "i"),
+            (float!(1.0), float!(10e-17), "1"),
+            (float!(10e-16), float!(0.0), "0"),
+            (float!(3.00000000004), float!(0.0), "3"),
         ];
         for (real, imaginary, output) in in_out {
-            let result =
-                KalkValue::Number(float!(real), float!(imaginary), None).to_string_pretty();
+            let result = KalkValue::Number(real, imaginary, None)
+                .to_string_pretty(ScientificNotationFormat::Normal);
             assert_eq!(output, result);
         }
     }
+
+    #[test]
+    fn test_eng_mode() {
+        let in_out = vec![
+            (1.23, 0, "1.23×10^0"),
+            (1.23, 1, "12.3×10^0"),
+            (1.23, 2, "123×10^0"),
+            (1.23, 3, "1.23×10^3"),
+            (1.23, 4, "12.3×10^3"),
+            (1.23, 5, "123×10^3"),
+            (1.23, 6, "1.23×10^6"),
+        ];
+        for (value, exponent, output) in in_out {
+            let sci = ScientificNotation {
+                value,
+                exponent: exponent + 1,
+                imaginary: false,
+            };
+
+            assert_eq!(
+                sci.to_string_format(ScientificNotationFormat::Engineering),
+                output
+            );
+        }
+    }
 }
