@@ -9,6 +9,14 @@ impl KalkValue {
         }
     }
 
+    pub fn to_float(&self) -> f64 {
+        self.to_f64()
+    }
+
+    pub fn imaginary_to_float(&self) -> f64 {
+        self.imaginary_to_f64()
+    }
+
     pub fn imaginary_to_f64(&self) -> f64 {
         if let KalkValue::Number(_, imaginary, _) = self {
             *imaginary
