@@ -4,6 +4,7 @@ use crate::analysis;
 use crate::ast::Identifier;
 use crate::calculation_result::CalculationResult;
 use crate::errors::KalkError;
+use crate::kalk_value::KalkFloat;
 use crate::{
     ast::{Expr, Stmt},
     interpreter,
@@ -122,8 +123,11 @@ pub fn eval(
 
     let result = interpreter.interpret(statements);
     if let Ok(Some(mut num)) = result {
-        num.set_radix(context.other_radix.unwrap_or(10));
-        Ok(Some(num))
+        if !num.set_radix(context.other_radix.unwrap_or(10)) {
+            Err(KalkError::InvalidBase)
+        } else {
+            Ok(Some(num))
+        }
     } else {
         result
     }
@@ -190,9 +194,9 @@ fn parse_piecewise(context: &mut Context) -> Result<Expr, KalkError> {
             advance(context);
             // Yeah, a bit hacky, but there's no `true` keyword...
             let true_expr = Expr::Binary(
-                Box::new(Expr::Literal(1f64)),
+                Box::new(Expr::Literal(crate::float!(1f64))),
                 TokenKind::Equals,
-                Box::new(Expr::Literal(1f64)),
+                Box::new(Expr::Literal(crate::float!(1f64))),
             );
             pieces.push(crate::ast::ConditionalPiece {
                 expr: left_expr,
@@ -329,7 +333,7 @@ fn parse_comparison(context: &mut Context) -> Result<Expr, KalkError> {
             context.symbol_table.get_mut().set(Stmt::FnDecl(
                 identifier.clone(),
                 parameters.clone(),
-                Box::new(Expr::Literal(1f64)),
+                Box::new(Expr::Literal(crate::float!(1f64))),
             ));
             let right = if match_token(context, TokenKind::OpenBrace) {
                 parse_piecewise(context)?
@@ -630,7 +634,16 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, KalkError> {
     let mut log_base = None;
     if identifier.full_name.starts_with("log") {
         if let Some(lowered) = identifier.get_lowered_part() {
-            if let Ok(lowered_float) = lowered.parse::<f64>() {
+            #[cfg(feature = "rug")]
+            fn parse_float_from_str(s: &str) -> Option<rug::Float> {
+                rug::Float::parse(s).map(|valid| crate::float!(valid)).ok()
+            }
+
+            #[cfg(not(feature = "rug"))]
+            fn parse_float_from_str(s: &str) -> Option<f64> {
+                s.parse::<f64>().ok()
+            }
+            if let Some(lowered_float) = parse_float_from_str(lowered) {
                 log_base = Some(Expr::Literal(lowered_float));
             }
         }
@@ -758,10 +771,25 @@ fn skip_newlines(context: &mut Context) {
     }
 }
 
-fn string_to_num(value: &str) -> Result<f64, KalkError> {
+fn string_to_num(value: &str) -> Result<KalkFloat, KalkError> {
+    #[cfg(feature = "rug")]
+    use rug::ops::Pow;
+
+    if value.contains('E') {
+        let parts = value.split('E').collect::<Vec<_>>();
+        let left = crate::float!(string_to_num(parts[0])?);
+        let right = crate::float!(string_to_num(parts[1])?);
+
+        #[cfg(feature = "rug")]
+        return Ok(left * 10.pow(right));
+
+        #[cfg(not(feature = "rug"))]
+        return Ok(left * 10_f64.powf(right));
+    }
+
     let base = get_base(value)?;
     if let Some(result) = crate::radix::parse_float_radix(&value.replace(' ', ""), base) {
-        Ok(result)
+        Ok(crate::float!(result))
     } else {
         Err(KalkError::InvalidNumberLiteral(value.into()))
     }
@@ -828,11 +856,11 @@ mod tests {
         let mut context = Context::new();
         context.symbol_table.get_mut().insert(Stmt::VarDecl(
             Identifier::from_full_name("x"),
-            literal(1f64),
+            f64_to_float_literal(1f64),
         ));
         context.symbol_table.get_mut().insert(Stmt::VarDecl(
             Identifier::from_full_name("y"),
-            literal(2f64),
+            f64_to_float_literal(2f64),
         ));
 
         // xy²
@@ -848,7 +876,7 @@ mod tests {
             Stmt::Expr(binary(
                 var("x"),
                 Star,
-                binary(var("y"), Power, literal(2f64))
+                binary(var("y"), Power, f64_to_float_literal(2f64))
             ))
         );
     }
@@ -875,15 +903,19 @@ mod tests {
         assert_eq!(
             parse(tokens).unwrap(),
             Stmt::Expr(binary(
-                literal(1f64),
+                f64_to_float_literal(1f64),
                 Plus,
                 binary(
-                    literal(2f64),
+                    f64_to_float_literal(2f64),
                     Star,
                     group(binary(
-                        literal(3f64),
+                        f64_to_float_literal(3f64),
                         Minus,
-                        binary(literal(4f64), Slash, literal(5f64))
+                        binary(
+                            f64_to_float_literal(4f64),
+                            Slash,
+                            f64_to_float_literal(5f64)
+                        )
                     ))
                 )
             ))
@@ -910,16 +942,20 @@ mod tests {
             parse(tokens).unwrap(),
             Stmt::Expr(binary(
                 binary(
-                    literal(1f64),
+                    f64_to_float_literal(1f64),
                     Star,
                     binary(
-                        literal(2f64),
+                        f64_to_float_literal(2f64),
                         Power,
-                        binary(literal(3f64), Power, literal(4f64)),
+                        binary(
+                            f64_to_float_literal(3f64),
+                            Power,
+                            f64_to_float_literal(4f64)
+                        ),
                     ),
                 ),
                 Plus,
-                literal(5f64)
+                f64_to_float_literal(5f64)
             )),
         );
     }
@@ -936,7 +972,11 @@ mod tests {
 
         assert_eq!(
             parse(tokens).unwrap(),
-            Stmt::Expr(binary(literal(10f64), Power, unary(Minus, literal(1f64)))),
+            Stmt::Expr(binary(
+                f64_to_float_literal(10f64),
+                Power,
+                unary(Minus, f64_to_float_literal(1f64))
+            )),
         );
     }
 
@@ -956,9 +996,13 @@ mod tests {
         assert_eq!(
             parse(tokens).unwrap(),
             Stmt::Expr(binary(
-                binary(literal(1f64), Percent, literal(1f64)),
+                binary(
+                    f64_to_float_literal(1f64),
+                    Percent,
+                    f64_to_float_literal(1f64)
+                ),
                 Plus,
-                unary(Percent, literal(5f64))
+                unary(Percent, f64_to_float_literal(5f64))
             ))
         );
     }
@@ -976,7 +1020,7 @@ mod tests {
 
         assert_eq!(
             parse_with_context(&mut context, tokens).unwrap(),
-            Stmt::Expr(unit("a", literal(1f64)))
+            Stmt::Expr(unit("a", f64_to_float_literal(1f64)))
         );
     }
 
@@ -996,7 +1040,7 @@ mod tests {
             parse(tokens).unwrap(),
             Stmt::VarDecl(
                 Identifier::from_full_name("x"),
-                binary(literal(1f64), Plus, literal(2f64))
+                binary(f64_to_float_literal(1f64), Plus, f64_to_float_literal(2f64))
             )
         );
     }
@@ -1021,7 +1065,7 @@ mod tests {
             Stmt::FnDecl(
                 Identifier::from_full_name("f"),
                 vec![String::from("f-x")],
-                binary(literal(1f64), Plus, param_var("f", "x"))
+                binary(f64_to_float_literal(1f64), Plus, param_var("f", "x"))
             )
         );
     }
@@ -1047,7 +1091,7 @@ mod tests {
         context.symbol_table.get_mut().set(Stmt::FnDecl(
             Identifier::from_full_name("f"),
             vec![String::from("x")],
-            literal(1f64),
+            f64_to_float_literal(1f64),
         ));
 
         assert_eq!(
@@ -1055,10 +1099,14 @@ mod tests {
             Stmt::Expr(binary(
                 Box::new(Expr::FnCall(
                     Identifier::from_full_name("f"),
-                    vec![*binary(literal(1f64), Plus, literal(2f64))]
+                    vec![*binary(
+                        f64_to_float_literal(1f64),
+                        Plus,
+                        f64_to_float_literal(2f64)
+                    )]
                 )),
                 Plus,
-                literal(3f64)
+                f64_to_float_literal(3f64)
             ))
         );
     }
