@@ -133,6 +133,7 @@ pub struct BinaryFuncInfo(
     FuncType,
 );
 
+#[allow(dead_code)]
 pub struct VectorFuncInfo(fn(KalkValue) -> Result<KalkValue, KalkError>, FuncType);
 
 impl UnaryFuncInfo {
@@ -242,7 +243,7 @@ fn to_angle_unit(context: &mut interpreter::Context, x: KalkValue, angle_unit: &
         "rad" => x,
         _ => interpreter::convert_unit(
             context,
-            &Expr::Literal(x.to_f64()),
+            &Expr::Literal(x.to_float()),
             Some(&String::from("rad")),
             Some(&angle_unit.to_string()),
         )
@@ -259,7 +260,7 @@ fn from_angle_unit(
         "rad" => x,
         _ => interpreter::convert_unit(
             context,
-            &Expr::Literal(x.to_f64()),
+            &Expr::Literal(x.to_float()),
             Some(&angle_unit.to_string()),
             Some(&String::from("rad")),
         )
